!-------------------------------------------------------------------------------

!                      Code_Saturne version 6.3-alpha
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2020 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
! Purpose:
! -------

!> \file cs_user_extra_operations.f90
!>
!> \brief This function is called at the end of each time step, and has a very
!>  general purpose
!>  (i.e. anything that does not have another dedicated user subroutine)
!>
!> See \ref cs_user_extra_operations_examples and
!> \ref cs_user_extra_operations-nusselt_calculation for examples.
!>
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine cs_f_user_extra_operations &
 ( nvar   , nscal  ,                                              &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use dimens, only: ndimfb
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use lagran
use parall
use period
use ppppar
use ppthch
use ppincl
use mesh
use field
use field_operator
use turbomachinery
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

double precision dt(ncelet)

! Local variables

!< [loc_var_dec]
integer          iel    , ifac
integer          ilelt  , nlelt, impout, iflmab

integer, allocatable, dimension(:) :: lstelt

double precision flumab_Blowing, pressure_Blowing, surface_Blowing
double precision flumab_Extraction, pressure_Extraction, surface_Extraction

double precision, dimension(:), pointer :: bmasfl, pressure

! Allocate a temporary array for cells or interior/boundary faces selection
allocate(lstelt(max(ncel,nfac,nfabor)))

call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
call field_get_val_s(iflmab, bmasfl)
call field_get_val_s(ipr, pressure)


flumab_Blowing = 0.d0
pressure_Blowing = 0.d0
surface_Blowing = 0.d0

call getfbr('Blowing', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)
  iel  = ifabor(ifac)   ! associated boundary cell

  ! Physical variables

  flumab_Blowing = flumab_Blowing + bmasfl(ifac)
  pressure_Blowing = pressure_Blowing + pressure(iel)*surfbn(ifac)
  surface_Blowing = surface_Blowing + surfbn(ifac)

enddo
  
flumab_Extraction = 0.d0
pressure_Extraction = 0.d0
surface_Extraction = 0.d0

call getfbr('Extraction', nlelt, lstelt)

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)
  iel  = ifabor(ifac)   ! associated boundary cell

  ! Physical variables

  flumab_Extraction = flumab_Extraction + bmasfl(ifac)
  pressure_Extraction = pressure_Extraction + pressure(iel)*surfbn(ifac)
  surface_Extraction = surface_Extraction + surfbn(ifac)

enddo
  
if (irangp.ge.0) then
  call parsom(flumab_Blowing)
  call parsom(flumab_Extraction)
  call parsom(pressure_Blowing)
  call parsom(pressure_Extraction)
  call parsom(surface_Blowing)
  call parsom(surface_Extraction)
endif

pressure_Blowing = pressure_Blowing/surface_Blowing
pressure_Extraction = pressure_Extraction/surface_Extraction

impout = impusr(1)
if (irangp.le.0) then
  open(file="Extraction.dat",unit=impout,position="append")
  write(impout,'(4E17.9)') ttcabs, pressure_Extraction + p0, pther, flumab_Extraction
  close(impout)
endif

impout = impusr(2)
if (irangp.le.0) then
  open(file="Blowing.dat",unit=impout,position="append")
  write(impout,'(4E17.9)') ttcabs, pressure_Blowing + p0, pther, flumab_Blowing
  close(impout)
endif

! Deallocate the temporary array
deallocate(lstelt)
!----
! End
!----

return
end subroutine cs_f_user_extra_operations
