!-------------------------------------------------------------------------------

!                      Code_Saturne version 6.3-alpha
!                      --------------------------
! This file is part of Code_Saturne, a general-purpose CFD tool.
!
! Copyright (C) 1998-2020 EDF S.A.
!
! This program is free software; you can redistribute it and/or modify it under
! the terms of the GNU General Public License as published by the Free Software
! Foundation; either version 2 of the License, or (at your option) any later
! version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
! FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
! details.
!
! You should have received a copy of the GNU General Public License along with
! this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
! Street, Fifth Floor, Boston, MA 02110-1301, USA.

!-------------------------------------------------------------------------------

!===============================================================================
!> \file cs_user_physical_properties.f90
!>
!> \brief Definition of physical variable laws.
!>
!> See \ref physical_properties for examples.
!>

!===============================================================================
!> \brief Definition of physical variable laws.
!>
!> \section Warning
!>
!> It is \b forbidden to modify turbulent viscosity \c visct here
!> (a specific subroutine is dedicated to that: \ref usvist)
!>
!> - icp = 0 must <b> have been specified </b>
!>    in \ref usipsu if we wish to define a variable specific heat
!>    cpro_cp (otherwise: memory overwrite).
!>
!> - the kivisl field integer key (diffusivity_id)
!>    must <b> have been specified </b>
!>    in \ref usipsu if we wish to define a variable viscosity
!>    \c viscls.
!>
!>
!> \remarks
!>  - This routine is called at the beginning of each time step
!>    Thus, <b> AT THE FIRST TIME STEP </b> (non-restart case), the only
!>    values initialized before this call are those defined
!>      - in the GUI or  \ref usipsu (cs_user_parameters.f90)
!>             - density    (initialized at \c ro0)
!>             - viscosity  (initialized at \c viscl0)
!>      - in the GUI or \ref cs_user_initialization
!>             - calculation variables (initialized at 0 by defaut
!>             or to the value given in the GUI or in \ref cs_user_initialization)
!>
!>  - We may define here variation laws for cell properties, for:
!>     - density:                                    rom    kg/m3
!>     - density at boundary faces:                  romb   kg/m3)
!>     - molecular viscosity:                        cpro_viscl  kg/(m s)
!>     - specific heat:                              cpro_cp     J/(kg degrees)
!>     - diffusivities associated with scalars:      cpro_vscalt kg/(m s)
!>
!> \b Warning: if the scalar is the temperature, cpro_vscalt corresponds
!> to its conductivity (Lambda) in W/(m K)
!>
!>
!> The types of boundary faces at the previous time step are available
!>   (except at the first time step, where arrays \c itypfb and \c itrifb have
!>   not been initialized yet)
!>
!> It is recommended to keep only the minimum necessary in this file
!>   (i.e. remove all unused example code)
!>
!>
!> \section usphyv_cell_id Cells identification
!>
!> Cells may be identified using the \ref getcel subroutine.
!> The syntax of this subroutine is described in the
!> \ref cs_user_boundary_conditions subroutine,
!> but a more thorough description can be found in the user guide.
!
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Arguments
!______________________________________________________________________________.
!  mode           name          role                                           !
!______________________________________________________________________________!
!> \param[in]     nvar          total number of variables
!> \param[in]     nscal         total number of scalars
!> \param[in]     mbrom         indicator of filling of romb array
!> \param[in]     dt            time step (per cell)
!_______________________________________________________________________________

subroutine usphyv &
 ( nvar   , nscal  ,                                              &
   mbrom  ,                                                       &
   dt     )

!===============================================================================

!===============================================================================
! Module files
!===============================================================================

use paramx
use pointe
use numvar
use optcal
use cstphy
use cstnum
use entsor
use parall
use period
use ppppar
use ppthch
use ppincl
use field
use mesh
use lagran
use cs_c_bindings

!===============================================================================

implicit none

! Arguments

integer          nvar   , nscal

integer          mbrom

double precision dt(ncelet)

!===============================================================================

! Local variables
integer          iel, ifac, iscal, ivslaw, ifcvsl
integer          iflmab

double precision varam, varbm, varcm, vardm
double precision xrtp, Tsuthe, temp
double precision turb_schmidt

integer          n, l
double precision som1, som2
double precision viscoefk(4,ngaze),visk(ngaze)
double precision yi(ngazg), yk(ngaze), xk(ngaze)

double precision mtot, mtota, flutot, rho, rhoa, alphaP

double precision, dimension(:), pointer :: crom, croma, brom
double precision, dimension(:), pointer :: bmasfl

double precision, dimension(:), pointer :: cpro_viscl, cpro_visls
double precision, dimension(:), pointer :: cpro_temp, cpro_yf, cpro_yo, cpro_yp

!===============================================================================

! Density
call field_get_val_s(icrom, crom)
call field_get_val_prev_s(icrom, croma)
call field_get_val_s(ibrom, brom)

! Mass fluxes
call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
call field_get_val_s(iflmab, bmasfl)

!===================================================================
! Variable viscosity as a function of temperature
!===================================================================

if (ivivar.eq.1) then

  call field_get_val_s(iprpfl(itemp) , cpro_temp)
  call field_get_val_s(iprpfl(iviscl), cpro_viscl)
  call field_get_val_s(iprpfl(iym(1)), cpro_yf)
  call field_get_val_s(iprpfl(iym(2)), cpro_yo)
  call field_get_val_s(iprpfl(iym(3)), cpro_yp)

  ! ivslaw = 10 fit Hanbook for air
  ! ivslaw = 11 Sutherland's law
  ! ivslaw = 20 mixing law

  ivslaw = 11

  ! fit Handbook for air
  if (ivslaw.eq.10) then

    do iel = 1, ncel

      varam = 46.291D0 * 1.D-7
      varbm = 0.52546D0 * 1.D-7
      varcm =-1.9103D-4 * 1.D-7
      vardm = 4.4842D-8 * 1.D-7

      xrtp = cpro_temp(iel)
      cpro_viscl(iel) =                                        &
      varam + xrtp*varbm + varcm*xrtp**2 + vardm*xrtp**3

    enddo

  ! Sutherland's law
  elseif (ivslaw.eq.11) then

    Tsuthe = 110.4d0

    do iel = 1, ncel

      xrtp = cpro_temp(iel)
      cpro_viscl(iel) = viscl0 *(Tkelvi+Tsuthe)/(xrtp+Tsuthe)*(xrtp/Tkelvi)**(3.d0/2.d0)

    enddo

  ! Mixing law
  elseif (ivslaw.eq.20) then

    ! ! c2h4
    ! viscoefk(1,1)= -3.583724530538e-007
    ! viscoefk(2,1)=  3.868415673467e-008
    ! viscoefk(3,1)= -1.193808567543e-011
    ! viscoefk(4,1)=  2.113573044333e-015
    ! c3h8
    viscoefk(1,1)= -1.234349675612e-007
    viscoefk(2,1)=  3.104728049493e-008
    viscoefk(3,1)= -9.675403508033e-012
    viscoefk(4,1)=  1.725116880193e-015
    ! o2
    viscoefk(1,2)=  5.488554654900e-006
    viscoefk(2,2)=  5.699109311534e-008
    viscoefk(3,2)= -1.793899193773e-011
    viscoefk(4,2)=  3.451533345962e-015
    ! co2
    viscoefk(1,3)=  2.817125838535e-007
    viscoefk(2,3)=  5.513314232221e-008
    viscoefk(3,3)=  -1.749297867996e-011
    viscoefk(4,3)=  3.166176962235e-015
    ! h2o
    viscoefk(1,4)= -2.754263332602e-006
    viscoefk(2,4)=  5.396477445927e-008
    viscoefk(3,4)= -9.502751904262e-012
    viscoefk(4,4)=  9.232335790842e-016
    ! n2
    viscoefk(1,5)= 5.181365609879e-006
    viscoefk(2,5)= 4.841826612088e-008
    viscoefk(3,5)= -1.490782620291e-011
    viscoefk(4,5)=  2.849448433980e-015

    do iel = 1, ncel

      som1=0.d0
      som2=0.d0
      visk=0.d0

      temp  = cpro_temp(iel)
      yi(1) = cpro_yf(iel)
      yi(2) = cpro_yo(iel)
      yi(3) = cpro_yp(iel)

      ! Calcul de la fraction molaire de chaque espece
      call yg2xye (yi, yk, xk)

      ! Calcul de la viscosite de chaque espece
      do n=1,ngaze
        do l=1,4
         visk(n)=visk(n)+viscoefk(l,n)*temp**(l-1)
        enddo
      enddo

      ! Calcul de la viscosite du melange
      do n=1,ngaze
        som1=som1+visk(n)*xk(n)
        som2=som2+xk(n)/visk(n)
      enddo

      cpro_viscl(iel) = 0.5d0*(som1+1.d0/som2)

    ENDDO

  endif

endif

!===================================================================
! Diffusivity as a function of viscosity for scalars
!===================================================================

call field_get_key_int(ivarfl(isca(ifm)), kivisl, ifcvsl)

if (ifcvsl.ge.0) then

  do iscal = 1, nscal

    ! We only handle here variables which are not fluctuations
    if (iscavr(iscal).le.0) then

      call field_get_val_s(iviscl, cpro_viscl)
      call field_get_key_int(ivarfl(isca(iscal)), kivisl, ifcvsl)
      if (ifcvsl.ge.0) then
        call field_get_val_s(ifcvsl, cpro_visls)
      else
        cpro_visls => NULL()
        write(nfecra,1010) iscal
        call csexit (1)
      endif

      call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)

      ! viscosity in kg/(m.s) at cell centers
      do iel = 1, ncel
        cpro_visls(iel) = cpro_viscl(iel)/turb_schmidt
      enddo

    endif

  enddo

endif


!===============================================================================

!--------
! Formats
!--------

#if defined(_CS_LANG_FR)

 1010 format(                                                     &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/,&
'@ @@ ATTENTION : ARRET LORS DU CALCUL DES GRANDEURS PHYSIQUES',/,&
'@    =========                                               ',/,&
'@    DONNEES DE CALCUL INCOHERENTES                          ',/,&
'@                                                            ',/,&
'@    Pour le scalaire ', i10                                  ,/,&
'@      la diffusivite est uniforme alors que                 ',/,&
'@      usphyv impose une diffusivite variable.               ',/,&
'@                                                            ',/,&
'@    Le calcul ne sera pas execute.                          ',/,&
'@                                                            ',/,&
'@    Modifier usipsu ou usphyv.                              ',/,&
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@                                                            ',/)

#else

 1010 format(                                                     &
'@',/,                                                            &
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@',/,                                                            &
'@ @@ WARNING:  stop when computing physical quantities',/,       &
'@    =======',/,                                                 &
'@    Inconsistent calculation data',/,                           &
'@',/,                                                            &
'@    For scalar', i10,/,                                         &
'@      the diffusivity is uniform while',/,                      &
'@      usphyv prescribes a variable diffusivity.',/,             &
'@',/,                                                            &
'@    The calculation will not be run.',/,                        &
'@',/,                                                            &
'@    Modify usipsu or usphyv.',/,                                &
'@                                                            ',/,&
'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
'@',/)

#endif

!----
! End
!----

return
end subroutine usphyv


